// -*- C++ -*-
/*!
 * @file  OutSpeakWave.cpp
 * @brief Wave Play Component
 * @date $Date$
 *
 * $Id$
 */

#include "OutSpeakWave.h"
#include <mmsystem.h>

#pragma comment(lib, "Winmm.lib")

// Module specification
// <rtc-template block="module_spec">
static const char* outspeakwave_spec[] =
  {
    "implementation_id", "OutSpeakWave",
    "type_name",         "OutSpeakWave",
    "description",       "Wave Play Component",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
OutSpeakWave::OutSpeakWave(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inSpeakDataIn("inSpeakData", m_inSpeakData)
    // </rtc-template>
{
	m_lpData = new char[8096];
	m_nSize = 8096;
}

/*!
 * @brief destructor
 */
OutSpeakWave::~OutSpeakWave()
{
	delete [] m_lpData;
}



RTC::ReturnCode_t OutSpeakWave::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inSpeakData", m_inSpeakDataIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  return RTC::RTC_OK;
}


RTC::ReturnCode_t OutSpeakWave::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t OutSpeakWave::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OutSpeakWave::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t OutSpeakWave::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "OutSpeakWave Start" << std::endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t OutSpeakWave::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout << "OutSpeakWave Stop" << std::endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t OutSpeakWave::onExecute(RTC::UniqueId ec_id)
{
  if(m_inSpeakDataIn.isNew())
  {
	m_inSpeakDataIn.read();

	std::string    inSpeakData;
	inSpeakData = m_inSpeakData.data;

    int size = (int)inSpeakData.size();
    if(m_nSize < size){
        delete [] m_lpData;
        m_nSize = size + 16;
        m_lpData = new char[m_nSize];
    }
    memcpy(m_lpData, inSpeakData.data(), size);
    char* sp = strstr(m_lpData, "<snd>");
    if(sp == NULL){
      std::cout << "ERROR1" << std::endl;
      return RTC::RTC_OK;
    }
    char* spTop = sp + strlen("<snd>");
    char* spEnd = strstr(m_lpData, "</snd>");
    sp = spEnd + strlen("</snd>");
    *spEnd      = 0x00;
    const char* wavedata = m_Base64.Decode((const char*)spTop);
    *spEnd      = '<';

	std::cout << "Speak" << std::endl;
	PlaySound(wavedata, NULL, SND_MEMORY | SND_SYNC);
	std::cout << "Done!" << std::endl;

    // `ԑf
    spTop = strstr(sp, "<mor>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</mor>");
      *spEnd = 0x00;
      std::cout << "mor!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // cabochao
    spTop = strstr(sp, "<cab>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</cab>");
      *spEnd = 0x00;
      std::cout << "cab!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // AAo
    spTop = strstr(sp, "<bxmor>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</bxmor>");
      *spEnd = 0x00;
      std::cout << "bxmor!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // |[Y
    spTop = strstr(sp, "<xmor>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</xmor>");
      *spEnd = 0x00;
      std::cout << "xmor!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // C
    spTop = strstr(sp, "<pli>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</pli>");
      *spEnd = 0x00;
      std::cout << "pli!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // 
    spTop = strstr(sp, "<stc>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</stc>");
      *spEnd = 0x00;
      std::cout << "stc!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // HTS x
    spTop = strstr(sp, "<lab>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</lab>");
      *spEnd = 0x00;
      std::cout << "lab!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // ^[Qbgfx
    spTop = strstr(sp, "<mk>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</mk>");
      *spEnd = 0x00;
      std::cout << "mk!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
    // ^[Qbg F0 
    spTop = strstr(sp, "<f0>");
    if(spTop != NULL)
    {
      spEnd = strstr(sp, "</f0>");
      *spEnd = 0x00;
      std::cout << "f0!" << std::endl;
      std::cout << spTop << std::endl;
      *spEnd = '<';
    }
  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t OutSpeakWave::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OutSpeakWave::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OutSpeakWave::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OutSpeakWave::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OutSpeakWave::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void OutSpeakWaveInit(RTC::Manager* manager)
  {
    coil::Properties profile(outspeakwave_spec);
    manager->registerFactory(profile,
                             RTC::Create<OutSpeakWave>,
                             RTC::Delete<OutSpeakWave>);
  }
  
};


